#!/usr/bin/perl
#!/usr/bin/perl -w

# Version 1.0
# December 18, 1996
# Written by Ken Lunde, Adobe Systems Incorporated (lunde@adobe.com)

require 5.002;

$vertical = $errorcount = 0;
@six_fields = (100, 50, 50, 100, 25, 25);

@names = qw[dfCopyright dfAscent dfInternalLeading dfWeight dfCharSet
	    dfPitchAndFamily etmCapHeight etmXHeight etmLowerCaseAscent
	    etmLowerCaseDescent WindowsName PSName Widths];

# Read in the contents of the data file, ignoring blank lines and non-
# key/value pairs

while($line = <STDIN>) {
    $line =~ s/^\s+//;
    $line =~ s/\s+$//;
    next unless ($key,$value) = split(/\s*=\s*/,$line,2) and defined $value;
    $pfm{lc $key} = $value;
}

# First, check to make sure that each required key exists

foreach $key (@names) {
    if (!exists $pfm{lc $key}) {
	print STDERR "$key is undefined!\n";
	$errorcount++;
    }
}
die "Exiting...\n" if $errorcount;

# If the WindowsName field is in an eight-bit printable format, convert
# it to true eight-bit. Two lowercase or uppercase hexadecimal digits,
# which represent a single character, can be prefixed with either an
# equals (=) or a percent (%) sign.

$pfm{windowsname} =~ s/[=%]([\dA-Fa-f]{2})/pack("C",hex($1))/eg;

# Calculate offsets and file size

$driverinfooffset = 210 + length($pfm{windowsname}) + 1;
$extenttbloffset = $driverinfooffset + length($pfm{psname}) + 1;
$size = $extenttbloffset + 448;

# Determine whether the font is vertical

if ($pfm{windowsname} =~ /^\@/) {
    $vertical = 1;
    @six_fields = (0, 0, 0, 0, 0, 0);
}

&MakeExtentTbl;

# Below are the binary packing structures for each section of the PFM.
# "v" is a little-endian two-byte value, "V" is a little-endian four-
# byte value, "a" is null-padded ASCII text, "C" is an unsigned char
# (one-byte), and "x" is a null byte.

$pat1 = "vVa60vvvvvvvCCCvCvvCvvCCCCvVVVV"; # PFM Header
$pat2 = "vVVVVVVV";			   # PFM Extension
$pat3 = "vvvvvvvvvvvvvvvvvvvvvvvvvv";	   # PFM Extended Text Metrics
$pat4 = "a*xa*xa*x";			   # PFM PostScript Information

# Each section is packed using the appropriate packing structure and
# values.

$string = pack($pat1,			   # Packing structure for PFM Header
	       256,			   # dfVersion
	       $size,			   # dfSize
	       $pfm{dfcopyright},	   # dfCopyright
	       129, 10, 300, 300,	   # dfType, dfPoint, dfVertRes, dfHorizRes
	       $pfm{dfascent},		   # dfAscent
	       $pfm{dfinternalleading},	   # dfInternalLeading
	       196,			   # dfExternalLeading
	       0, 0, 0,			   # dfItalic, dfUnderline, dfStrikeOut
	       $pfm{dfweight},		   # dfWeight
	       $pfm{dfcharset},		   # dfCharSet
	       0, 1000,			   # dfPixWidth, dfPixHeight
	       $pfm{dfpitchandfamily},	   # dfPitchAndFamily
	       500, 1000, 32, 255,	   # dfAvgWidth, dfMaxWidth, dfFirstChar, dfLastChar
	       0, 0, 0,			   # dfDefaultChar, dfBreakChar, dfWidthBytes
	       199, 210, 0, 0);		   # dfDevice, dfFace, dfBitsPointer, dfBitsOffset

$string .= pack($pat2,			   # Packing structure for PFM Extension
		30, 147,		   # dfSizeFields, dfExtMetricsOffset
		$extenttbloffset,	   # dfExtentTable
		0, 0, 0,		   # dfOriginTable, dfPairKernTable, dfTrackKernTable
		$driverinfooffset,	   # dfDriverInfo
		0);			   # dfReserved

$string .= pack($pat3,			   # Packing structure for PFM Extended Text Metrics
		52, 240, 0,		   # etmSize, etmPointSize, etmOrientation
		1000, 3, 1000,		   # etmMasterHeight, etmMinScale, etmMaxScale
		1000,			   # etmMasterUnits
		$pfm{etmcapheight},	   # etmCapHeight
		$pfm{etmxheight},	   # etmXHeight
		$pfm{etmlowercaseascent},  # etmLowerCaseAscent
		$pfm{etmlowercasedescent}, # etmLowerCaseDescent
		0, -500, 250,		   # etmSlant, etmSuperScript, etmSubScript
		500, 500,		   # etmSuperScriptSize, etmSubScriptSize
		@six_fields,		   # etmUnderlineOffset, etmUnderlineWidth,
					   # etmDoubleUpperUnderlineOffset,
					   # etmDoubleLowerUnderlineOffset,
					   # etmDoubleUpperUnderlineWidth,
					   # etmDoubleLowerUnderlineWidth
		405, 50,		   # etmStrikeOutOffset, etmStrikeOutWidth
		0, 0);			   # etmKernPairs, etmKernTracks

$string .= pack($pat4, "PostScript", $pfm{windowsname}, $pfm{psname});

foreach $key (sort {$a <=> $b} keys %extenttbl) {
    $string .= pack("v", $extenttbl{$key});
}

# Now, write the PFM to STDOUT

print STDOUT $string;

# The following function builds a database for creating the Extent Table,
# which provides explicit width information for the characters in the
# range 32 through 255, in device units.

sub MakeExtentTbl {
    foreach $element (32 .. 255) {
	$extenttbl{$element} = 500;
    }
    if ($pfm{widths} =~ /^\d+$/) {
	foreach $element (32 .. 126) {
	    $extenttbl{$element} = $pfm{widths};
	}
    } elsif ($pfm{widths} =~ /,/) {
	@widths = split(/\s*,\s*/,$pfm{widths});
	$count = 0;
	foreach $element (32 .. ($#widths + 32)) {
	    $extenttbl{$element} = $widths[$count];
	    $count++;
	}
    }
}
